#!/usr/bin/env python

# BSD 3-Clause License; see https://github.com/scikit-hep/uproot/blob/master/LICENSE

from os.path import join

import pytest
import numpy
import ctypes

import awkward

import uproot
from uproot.write.objects.TTree import newtree, newbranch

ROOT = pytest.importorskip("ROOT")

def test_strings(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename, compression=None) as f:
        f["hello"] = "world"

    f = ROOT.TFile.Open(filename)
    assert str(f.Get("hello")) == "world"
    f.Close()

def test_cycle(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename, compression=None) as f:
        f["hello"] = "world"
        f["hello"] = "uproot"

    f = ROOT.TFile.Open(filename)
    assert str(f.Get("hello;1")) == "world"
    assert str(f.Get("hello;2")) == "uproot"
    f.Close()

def test_zlib(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename, compression=uproot.ZLIB(1)) as f:
        f["hello"] = "a"*2000

    f = ROOT.TFile.Open(filename)
    assert f.GetCompressionAlgorithm() == uproot.const.kZLIB
    assert f.GetCompressionLevel() == 1
    assert str(f.Get("hello")) == "a"*2000
    f.Close()

def test_compresschange(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename, compression=uproot.ZLIB(2)) as f:
        f.compression = uproot.ZLIB(3)
        f["hello"] = "a"*2000

    f = ROOT.TFile.Open(filename)
    assert f.GetCompressionAlgorithm() == uproot.const.kZLIB
    assert f.GetCompressionLevel() == 3

def test_nocompress(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename, compression=None) as f:
        f["hello"] = "a"*2000

    f = ROOT.TFile.Open(filename)
    assert f.GetCompressionFactor() == 1
    assert str(f.Get("hello")) == "a"*2000
    f.Close()

def test_compress_small_data(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename, compression=uproot.ZLIB(4)) as f:
        f["hello"] = "a"

    f = ROOT.TFile.Open(filename)
    assert str(f.Get("hello")) == "a"
    f.Close()

def test_lzma(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename, compression=uproot.LZMA(1)) as f:
        f["hello"] = "a"*2000

    f = ROOT.TFile.Open(filename)
    assert f.GetCompressionAlgorithm() == uproot.const.kLZMA
    assert f.GetCompressionLevel() == 1
    assert str(f.Get("hello")) == "a"*2000
    f.Close()

def test_lz4_leveldown(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename, compression=uproot.LZ4(5)) as f:
        f["hello"] = "a"*2000

    f = ROOT.TFile.Open(filename)
    assert (f.GetCompressionAlgorithm()) == uproot.const.kLZ4
    assert str(f.Get("hello")) == "a"*2000
    f.Close()

def test_lz4_levelup(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename, compression=uproot.LZ4(5)) as f:
        f["hello"] = "a"*2000

    f = ROOT.TFile.Open(filename)
    assert (f.GetCompressionAlgorithm()) == uproot.const.kLZ4
    assert (f.GetCompressionLevel()) == 5
    assert str(f.Get("hello")) == "a"*2000
    f.Close()

def test_th1(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH1F("hvar", "title", 5, 1, 10)
    h.Sumw2()
    h.Fill(1.0, 3)
    h.Fill(2.0, 4)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    sums = [0.0, 25.0, 0.0, 0.0, 0.0, 0.0, 0.0]
    bincontents = [7.0, 0.0, 0.0, 0.0, 0.0]
    assert list(h.GetSumw2()) == sums
    count = 0
    for x in range(1, 6):
        assert h.GetBinContent(x) == bincontents[count]
        count += 1
    assert h.GetNbinsX() == 5
    assert h.GetMean() == 1.5714285714285714
    assert h.GetRMS() == 0.4948716593053938

def test_th1_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH1F("hvar", "title", 5, 1, 10)
    h.Sumw2()
    h.Fill(1.0, 3)
    h.Fill(2.0, 4)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    assert "TH1" in uproot.open(filename)["test"]._classname.decode("utf-8")

def test_th1_varbin(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    import numpy as np
    bins = np.array([1.0, 3.0, 4.0, 10.0, 11.0, 12.0], dtype="float64")
    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH1F("hvar", "title", 5, bins)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    assert h.GetNbinsX() == 5
    assert h.GetBinWidth(1) == 2.0
    assert h.GetBinWidth(2) == 1.0
    assert h.GetBinWidth(3) == 6.0
    assert h.GetBinWidth(4) == 1.0
    assert h.GetBinWidth(5) == 1.0

def test_compressed_th1(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    import numpy as np
    bins = np.array([1.0, 3.0, 4.0, 10.0, 11.0, 12.0], dtype="float64")
    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH1F("hvar", "title", 5, bins)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=uproot.ZLIB(1)) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    assert h.GetNbinsX() == 5
    assert h.GetBinWidth(1) == 2.0
    assert h.GetBinWidth(2) == 1.0
    assert h.GetBinWidth(3) == 6.0
    assert h.GetBinWidth(4) == 1.0
    assert h.GetBinWidth(5) == 1.0

def test_th2(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH2F("hvar", "title", 5, 1, 10, 6, 1, 20)
    h.Sumw2()
    h.Fill(1.0, 5.0, 3)
    h.Fill(2.0, 10.0, 4)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    sums = [0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 9.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 16.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0]
    bincontents = [0.0, 3.0, 4.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
    count = 0
    for x in range(1, 6):
        for y in range(1, 7):
            assert h.GetBinContent(x, y) == bincontents[count]
            count += 1
    assert list(h.GetSumw2()) == sums
    assert h.GetMean() == 1.5714285714285714
    assert h.GetRMS() == 0.4948716593053938
    assert h.GetNbinsX() == 5
    assert h.GetNbinsY() == 6

def test_th2_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH2F("hvar", "title", 5, 1, 10, 6, 1, 20)
    h.Sumw2()
    h.Fill(1.0, 5.0, 3)
    h.Fill(2.0, 10.0, 4)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    assert "TH2" in uproot.open(filename)["test"]._classname.decode("utf-8")

def test_th2_varbin(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    import numpy as np
    binsx = np.array([1.0, 3.0, 4.0, 10.0, 11.0, 12.0], dtype="float64")
    binsy = np.array([1.0, 3.0, 4.0, 10.0, 11.0, 12.0, 20.0], dtype="float64")
    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH2F("hvar", "title", 5, binsx, 6, binsy)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    assert h.GetNbinsX() == 5
    assert h.GetNbinsY() == 6

def test_compressed_th2(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    import numpy as np
    binsx = np.array([1.0, 3.0, 4.0, 10.0, 11.0, 12.0], dtype="float64")
    binsy = np.array([1.0, 3.0, 4.0, 10.0, 11.0, 12.0, 20.0], dtype="float64")
    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH2F("hvar", "title", 5, binsx, 6, binsy)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=uproot.ZLIB(1)) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    assert h.GetNbinsX() == 5
    assert h.GetNbinsY() == 6

def test_th3(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH3F("hvar", "title", 5, 1, 10, 6, 1, 20, 7, 1, 30)
    h.Sumw2()
    h.Fill(1.0, 5.0, 8.0, 3)
    h.Fill(2.0, 10.0, 9.0, 4)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    sums = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 9.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 16.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
    bincontents = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 3.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 4.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
    assert h.GetNbinsX() == 5
    assert h.GetNbinsY() == 6
    assert h.GetNbinsZ() == 7
    assert list(h.GetSumw2()) == sums
    assert h.GetMean() == 1.5714285714285714
    assert h.GetRMS() == 0.4948716593053938
    count = 0
    for x in range(1, 6):
        for y in range(1, 7):
            for z in range(1, 8):
                assert h.GetBinContent(x, y, z) == bincontents[count]
                count += 1

def test_th3_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH3F("hvar", "title", 5, 1, 10, 6, 1, 20, 7, 1, 30)
    h.Sumw2()
    h.Fill(1.0, 5.0, 8.0, 3)
    h.Fill(2.0, 10.0, 9.0, 4)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    assert "TH3" in uproot.open(filename)["test"]._classname.decode("utf-8")

def test_th3_varbin(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    import numpy as np
    binsx = np.array([1.0, 3.0, 4.0, 10.0, 11.0, 12.0], dtype="float64")
    binsy = np.array([1.0, 3.0, 4.0, 10.0, 11.0, 12.0, 20.0], dtype="float64")
    binsz = np.array([1.0, 10.0, 13.0, 14.0, 16.0, 20.0, 21.0, 23.0], dtype="float64")
    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH3F("hvar", "title", 5, binsx, 6, binsy, 7, binsz)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    assert h.GetNbinsX() == 5
    assert h.GetNbinsY() == 6
    assert h.GetNbinsZ() == 7

def test_compressed_th3(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    import numpy as np
    binsx = np.array([1.0, 3.0, 4.0, 10.0, 11.0, 12.0], dtype="float64")
    binsy = np.array([1.0, 3.0, 4.0, 10.0, 11.0, 12.0, 20.0], dtype="float64")
    binsz = np.array([1.0, 10.0, 13.0, 14.0, 16.0, 20.0, 21.0, 23.0], dtype="float64")
    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH3F("hvar", "title", 5, binsx, 6, binsy, 7, binsz)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=uproot.ZLIB(1)) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    assert h.GetNbinsX() == 5
    assert h.GetNbinsY() == 6
    assert h.GetNbinsZ() == 7

def test_tprofile(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TProfile("hvar", "title", 5, 1, 10)
    h.Sumw2()
    h.Fill(1.0, 3)
    h.Fill(2.0, 4)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    sums = [0.0, 25.0, 0.0, 0.0, 0.0, 0.0, 0.0]
    bincontents = [3.5, 0.0, 0.0, 0.0, 0.0]
    assert list(h.GetSumw2()) == sums
    assert h.GetMean() == 1.5
    assert h.GetRMS() == 0.5
    count = 0
    for x in range(1, 6):
        assert h.GetBinContent(x) == bincontents[count]
        count += 1

def test_tprofile_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TProfile("hvar", "title", 5, 1, 10)
    h.Sumw2()
    h.Fill(1.0, 3)
    h.Fill(2.0, 4)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    assert uproot.open(filename)["test"]._classname == b"TProfile"

def test_compressed_tprofile(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TProfile("hvar", "title", 5, 1, 10)
    h.Sumw2()
    h.Fill(1.0, 3)
    h.Fill(2.0, 4)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=uproot.LZMA(5)) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    sums = [0.0, 25.0, 0.0, 0.0, 0.0, 0.0, 0.0]
    bincontents = [3.5, 0.0, 0.0, 0.0, 0.0]
    assert list(h.GetSumw2()) == sums
    assert h.GetMean() == 1.5
    assert h.GetRMS() == 0.5
    count = 0
    for x in range(1, 6):
        assert h.GetBinContent(x) == bincontents[count]
        count += 1

def test_tprofile2d(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TProfile2D("hvar", "title", 5, 1, 10, 6, 1, 20)
    h.Sumw2()
    h.Fill(1.0, 5.0, 3)
    h.Fill(2.0, 10.0, 4)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    sums = [0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 9.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 16.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0]
    bincontents = [0.0, 3.0, 4.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
    count = 0
    for x in range(1, 6):
        for y in range(1, 7):
            assert h.GetBinContent(x, y) == bincontents[count]
            count += 1
    assert list(h.GetSumw2()) == sums
    assert h.GetMean() == 1.5
    assert h.GetRMS() == 0.5
    assert h.GetNbinsX() == 5
    assert h.GetNbinsY() == 6

def test_tprofile2d_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TProfile2D("hvar", "title", 5, 1, 10, 6, 1, 20)
    h.Sumw2()
    h.Fill(1.0, 5.0, 3)
    h.Fill(2.0, 10.0, 4)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    assert uproot.open(filename)["test"]._classname == b"TProfile2D"

def test_compressed_tprofile2d(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TProfile2D("hvar", "title", 5, 1, 10, 6, 1, 20)
    h.Sumw2()
    h.Fill(1.0, 5.0, 3)
    h.Fill(2.0, 10.0, 4)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=uproot.LZMA(5)) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    sums = [0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 9.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 16.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0 , 0.0]
    bincontents = [0.0, 3.0, 4.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
    count = 0
    for x in range(1, 6):
        for y in range(1, 7):
            assert h.GetBinContent(x, y) == bincontents[count]
            count += 1
    assert list(h.GetSumw2()) == sums
    assert h.GetMean() == 1.5
    assert h.GetRMS() == 0.5
    assert h.GetNbinsX() == 5
    assert h.GetNbinsY() == 6

def test_tprofile3d(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TProfile3D("hvar", "title", 5, 1, 10, 6, 1, 20, 8, 2, 8)
    h.Sumw2()
    h.Fill(1.0, 5.0, 3, 6)
    h.Fill(2.0, 10.0, 4, 7)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    sums = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 36.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 49.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
    bincontents = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
    count = 0
    for x in range(1, 6):
        for y in range(1, 7):
            for z in range(1, 9):
                assert h.GetBinContent(x, y, z) == bincontents[count]
                count += 1
    assert list(h.GetSumw2()) == sums
    assert h.GetMean() == 1.5
    assert h.GetRMS() == 0.5
    assert h.GetNbinsX() == 5
    assert h.GetNbinsY() == 6
    assert h.GetNbinsZ() == 8

def test_tprofile3d_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TProfile3D("hvar", "title", 5, 1, 10, 6, 1, 20, 8, 2, 5)
    h.Sumw2()
    h.Fill(1.0, 5.0, 3, 5)
    h.Fill(2.0, 10.0, 4, 8)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    assert uproot.open(filename)["test"]._classname == b"TProfile3D"

def test_compressed_tprofile3d(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TProfile3D("hvar", "title", 5, 1, 10, 6, 1, 20, 8, 2, 8)
    h.Sumw2()
    h.Fill(1.0, 5.0, 3, 6)
    h.Fill(2.0, 10.0, 4, 7)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=uproot.LZMA(6)) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    sums = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 36.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 49.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
    bincontents = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 6.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 7.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
    count = 0
    for x in range(1, 6):
        for y in range(1, 7):
            for z in range(1, 9):
                assert h.GetBinContent(x, y, z) == bincontents[count]
                count += 1
    assert list(h.GetSumw2()) == sums
    assert h.GetMean() == 1.5
    assert h.GetRMS() == 0.5
    assert h.GetNbinsX() == 5
    assert h.GetNbinsY() == 6
    assert h.GetNbinsZ() == 8

def test_dir_allocation(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename, compression=None) as f:
        for i in range(1, 101):
            f["a"*i] = "a"*i

    f = ROOT.TFile.Open(filename)
    assert str(f.Get("a"*100)) == "a"*100
    f.Close()

def test_taxis_axisbins(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH1F("hvar", "title", 5, 1, 10)
    h.Fill(1.0, 3)
    h.Fill(3.0, 8)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    assert h.GetXaxis().GetFirst() == 1
    assert h.GetXaxis().GetLast() == 5

def test_taxis_time(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH1F("hvar", "title", 5, 1, 10)
    h.GetXaxis().SetTimeDisplay(1)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    assert h.GetXaxis().GetTimeDisplay() == True

def test_th1_binlabel1(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH1F("hvar", "title", 5, 1, 10)
    h.Fill(1.0, 3)
    h.GetXaxis().SetBinLabel(1, "Hi")
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    assert h.GetXaxis().GetBinLabel(1) == "Hi"

def test_th1_binlabel1_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH1F("hvar", "title", 5, 1, 10)
    h.Fill(1.0, 3)
    h.GetXaxis().SetBinLabel(1, "Hi")
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = uproot.open(filename)
    h = f["test"]
    assert h._fXaxis._fLabels[0] == b"Hi"

def test_th1_binlabel2(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH1F("hvar", "title", 5, 1, 10)
    h.Fill(1.0, 3)
    h.Fill(2.0, 4)
    h.GetXaxis().SetBinLabel(1, "Hi")
    h.GetXaxis().SetBinLabel(2, "Hello")
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    assert h.GetXaxis().GetBinLabel(1) == "Hi"
    assert h.GetXaxis().GetBinLabel(2) == "Hello"

def test_th1_binlabel2_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH1F("hvar", "title", 5, 1, 10)
    h.Fill(1.0, 3)
    h.Fill(2.0, 4)
    h.GetXaxis().SetBinLabel(1, "Hi")
    h.GetXaxis().SetBinLabel(2, "Hello")
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = uproot.open(filename)
    h = f["test"]
    assert h._fXaxis._fLabels[0] == b"Hi"
    assert h._fXaxis._fLabels[1] == b"Hello"

def test_th2_binlabel1(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH2F("hvar", "title", 5, 1, 10, 6, 1, 20)
    h.Fill(1.0, 5.0, 3)
    h.GetXaxis().SetBinLabel(1, "Hi1")
    h.GetYaxis().SetBinLabel(2, "Hi2")
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    assert h.GetXaxis().GetBinLabel(1) == "Hi1"
    assert h.GetYaxis().GetBinLabel(2) == "Hi2"

def test_th3_binlabel1(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH3F("hvar", "title", 5, 1, 10, 6, 1, 20, 7, 1, 30)
    h.Fill(1.0, 5.0, 8.0, 3)
    h.GetXaxis().SetBinLabel(1, "Hi1")
    h.GetYaxis().SetBinLabel(2, "Hi2")
    h.GetZaxis().SetBinLabel(3, "Hi3")
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    assert h.GetXaxis().GetBinLabel(1) == "Hi1"
    assert h.GetYaxis().GetBinLabel(2) == "Hi2"
    assert h.GetZaxis().GetBinLabel(3) == "Hi3"

def test_objany_multihist(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH1F("hvar", "title", 5, 1, 10)
    h.Fill(1.0, 3)
    h.GetXaxis().SetBinLabel(1, "Hi")
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist
        f["test1"] = hist

    f = ROOT.TFile.Open(filename)
    h = f.Get("test")
    h1 = f.Get("test1")
    assert h.GetXaxis().GetBinLabel(1) == "Hi"
    assert h1.GetXaxis().GetBinLabel(1) == "Hi"

def test_objany_multihist_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH1F("hvar", "title", 5, 1, 10)
    h.Fill(1.0, 3)
    h.GetXaxis().SetBinLabel(1, "Hi")
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist
        f["test1"] = hist

    f = uproot.open(filename)
    h = f["test"]
    h1 = f["test1"]
    assert h._fXaxis._fLabels[0] == b"Hi"
    assert h1._fXaxis._fLabels[0] == b"Hi"

def test_ttree(tmp_path):
    filename = join(str(tmp_path), "example.root")

    tree = newtree()
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree

    f = ROOT.TFile.Open(filename)
    assert f.GetKey("t").GetClassName() == "TTree"

def test_tree_diff_interface(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename, compression=None) as f:
        f.newtree("t")

    f = ROOT.TFile.Open(filename)
    assert f.GetKey("t").GetClassName() == "TTree"

def test_ttree_multiple(tmp_path):
    filename = join(str(tmp_path), "example.root")

    tree = newtree()
    with uproot.recreate(filename, compression=None) as f:
        for i in range(100):
            f["t"*(i+1)] = tree

    f = ROOT.TFile.Open(filename)
    for i in range(100):
        assert f.GetKey("t"*(i+1)).GetClassName() == "TTree"

def test_ttree_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")

    tree = newtree()
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree

    f = uproot.open(filename)
    assert f["t"]._classname == b"TTree"

def test_ttree_multiple_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")

    tree = newtree()
    with uproot.recreate(filename, compression=None) as f:
        for i in range(100):
            f["t"*(i+1)] = tree

    f = uproot.open(filename)
    for i in range(100):
        assert f["t"*(i+1)]._classname == b"TTree"

def test_ttree_empty_tbranch(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree

    f = ROOT.TFile.Open(filename)
    assert f.Get("t").GetBranch("intBranch").GetName() == "intBranch"

def test_ttree_empty_tbranch_multitree(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    with uproot.recreate(filename, compression=None) as f:
        for i in range(10):
            f["t" * (i + 1)] = tree

    f = ROOT.TFile.Open(filename)
    for i in range(10):
        assert f.Get("t" * (i + 1)).GetBranch("intBranch").GetName() == "intBranch"

def test_ttree_empty_tbranch_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree

    f = uproot.open(filename)
    assert f["t"]["intBranch"]._classname == b"TBranch"

def test_ttree_empty_tbranch_multitree_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    with uproot.recreate(filename, compression=None) as f:
        for i in range(10):
            f["t"*(i+1)] = tree

    f = uproot.open(filename)
    for i in range(10):
        assert f["t" * (i + 1)]["intBranch"]._classname == b"TBranch"

def test_ttree_empty_tbranch_multiple(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b, "testbranch": b}
    tree = newtree(branchdict)
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree

    f = ROOT.TFile.Open(filename)
    assert f.Get("t").GetBranch("intBranch").GetName() == "intBranch"
    assert f.Get("t").GetBranch("testbranch").GetName() == "testbranch"

def test_ttree_empty_tbranch_multiple_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b, "testbranch": b}
    tree = newtree(branchdict)
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree

    f = uproot.open(filename)
    assert f["t"]["intBranch"]._classname == b"TBranch"
    assert f["t"]["testbranch"]._classname == b"TBranch"

def test_ttree_empty_tbranch_diff_type(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int64")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree

    f = ROOT.TFile.Open(filename)
    assert f.Get("t").GetBranch("intBranch").GetName() == "intBranch"

def test_ttree_empty_tbranch_title(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32", title="hi")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree

    f = ROOT.TFile.Open(filename)
    assert f.Get("t").GetBranch("intBranch").GetTitle() == "hi"

def test_hist_rewrite_root(tmp_path):
    filename = join(str(tmp_path), "example.root")
    testfile = join(str(tmp_path), "test.root")

    f = ROOT.TFile.Open(testfile, "RECREATE")
    h = ROOT.TH1F("hvar", "title", 5, 1, 10)
    h.Sumw2()
    h.Fill(1.0, 3)
    h.Fill(2.0, 4)
    h.Write()
    f.Close()

    t = uproot.open(testfile)
    hist = t["hvar"]
    with uproot.recreate(filename, compression=None) as f:
        f["test"] = hist

    f = ROOT.TFile.Open(filename, "UPDATE")
    t = ROOT.TObjString("Hello World")
    t.Write()
    f.Close()

    f = ROOT.TFile.Open(filename)
    assert f.Get("Hello World") == "Hello World"

def test_empty_ttree_rewrite_root(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree

    f = ROOT.TFile.Open(filename, "UPDATE")
    t = ROOT.TObjString("Hello World")
    t.Write()
    f.Close()

    f = ROOT.TFile.Open(filename)
    assert f.Get("Hello World") == "Hello World"

def test_string_rewrite_root(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename, compression=None) as f:
        f["a"*5] = "a"*5

    f = ROOT.TFile.Open(filename, "UPDATE")
    t = ROOT.TObjString("Hello World")
    t.Write()
    f.Close()

    f = ROOT.TFile.Open(filename)
    assert f.Get("Hello World") == "Hello World"

def test_string_rewrite_root_compress(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename, compression=uproot.ZLIB(4)) as f:
        f["a"*5] = "a"*5

    f = ROOT.TFile.Open(filename, "UPDATE")
    t = ROOT.TObjString("Hello World")
    t.Write()
    f.Close()

    f = ROOT.TFile.Open(filename)
    assert f.Get("Hello World") == "Hello World"

def test_branch_alt_interface(tmp_path):
    filename = join(str(tmp_path), "example.root")

    branchdict = {"intBranch": "int"}
    tree = newtree(branchdict)
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree

    f = ROOT.TFile.Open(filename)
    assert f.Get("t").GetBranch("intBranch").GetName() == "intBranch"

def test_branch_basket_one(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype=">i4")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">i4")
    for i in range(5):
        assert a[i] == treedata[i]

def test_branch_basket_one_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5]).astype("int32").newbyteorder(">")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)

    f = uproot.open(filename)
    tree = f["t"]
    treedata = tree.array("intBranch")
    for i in range(5):
        assert a[i] == treedata[i]

def test_branch_basket_one_rewrite_root(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5]).astype("int32").newbyteorder(">")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)

    f = ROOT.TFile.Open(filename, "UPDATE")
    t = ROOT.TObjString("Hello World")
    t.Write()
    f.Close()

    f = ROOT.TFile.Open(filename)
    assert f.Get("Hello World") == "Hello World"

def test_branch_basket_one_more_data(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = []
    for i in range(0, 100):
        a.append(i)
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">i4")
    for i in range(0, 100):
        assert a[i] == treedata[i]

def test_branch_basket_one_less_data(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4], dtype=">i4")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">i4")
    for i in range(4):
        assert a[i] == treedata[i]

def test_branch_basket_one_tleafb(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int8")
    branchdict = {"int8Branch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype="int8")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["int8Branch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype("int8")
    for i in range(5):
        assert a[i] == treedata[i]

def test_branch_basket_one_tleaff(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("float32")
    branchdict = {"floatBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype="float32")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["floatBranch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype("float32")
    for i in range(5):
        assert a[i] == treedata[i]

def test_branch_basket_one_tleafd(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch(">f8")
    branchdict = {"float8Branch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype=">f8")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["float8Branch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">f8")
    for i in range(5):
        assert a[i] == treedata[i]

def test_branch_basket_one_tleafl(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch(">i8")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype=">i8")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">i8")
    for i in range(5):
        assert a[i] == treedata[i]

def test_branch_basket_one_tleafO(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch(">?")
    branchdict = {"booleanBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 0, 0, 0, 1], dtype=">?")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["booleanBranch"].newbasket(a)

    ROOT.gInterpreter.Declare("""
    void readnewbasket(bool* arr, char* filename) {
        Bool_t x;
        TFile f(filename);
        auto tree = f.Get<TTree>("t");
        auto branch = tree->GetBranch("booleanBranch");
        branch->SetAddress(&x);
        for (int i=0; i<tree->GetEntries(); i++) {
            tree->GetEvent(i);
            arr[i] = x;
        }
    }""")

    testa = numpy.array([0, 0, 0, 0, 0], dtype=">?")
    ROOT.readnewbasket(testa, filename)
    for i in range(5):
        assert a[i] == testa[i]

def test_branch_basket_one_tleafs(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch(">i2")
    branchdict = {"int2Branch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype=">i2")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["int2Branch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">i2")
    for i in range(5):
        assert a[i] == treedata[i]

def test_one_branch_multi_basket(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype=">i4")
    b = numpy.array([6, 7, 8, 9, 10], dtype=">i4")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)
        f["t"]["intBranch"].newbasket(b)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">i4")
    for i in range(5):
        assert a[i] == treedata[i]
        assert b[i] == treedata[i+5]

def test_multi_branch_one_basket_same_type(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b1 = newbranch("int32")
    b2 = newbranch("int32")
    branchdict = {"intBranch": b1, "intBranch2": b2}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype=">i4")
    b = numpy.array([6, 7, 8, 9, 10], dtype=">i4")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)
        f["t"]["intBranch2"].newbasket(b)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    intBranchdata = tree.AsMatrix(["intBranch"]).astype(">i4")
    int8Branchdata = tree.AsMatrix(["intBranch2"]).astype(">i4")
    for i in range(5):
        assert a[i] == intBranchdata[i]
        assert b[i] == int8Branchdata[i]

def test_multi_branch_one_basket_same_type_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b1 = newbranch("int32")
    b2 = newbranch("int32")
    branchdict = {"intBranch": b1, "intBranch2": b2}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype=">i4")
    b = numpy.array([6, 7, 8, 9, 10], dtype=">i4")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)
        f["t"]["intBranch2"].newbasket(b)

    f = uproot.open(filename)
    tree = f["t"]
    intBranchdata = tree.array("intBranch")
    int8Branchdata = tree.array("intBranch2")
    for i in range(5):
        assert a[i] == intBranchdata[i]
        assert b[i] == int8Branchdata[i]

def test_multi_branch_one_basket_diff_type(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b1 = newbranch("int32")
    b2 = newbranch("int64")
    branchdict = {"intBranch": b1, "int8Branch": b2}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype=">i4")
    b = numpy.array([6, 7, 8, 9, 10], dtype=">i8")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)
        f["t"]["int8Branch"].newbasket(b)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    intBranchdata = tree.AsMatrix(["intBranch"]).astype(">i4")
    int8Branchdata = tree.AsMatrix(["int8Branch"]).astype(">i8")
    for i in range(5):
        assert a[i] == intBranchdata[i]
        assert b[i] == int8Branchdata[i]

def test_multi_branch_multi_basket_diff_type(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b1 = newbranch("int32")
    b2 = newbranch("int64")
    branchdict = {"intBranch": b1, "int8Branch": b2}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype=">i4")
    b = numpy.array([6, 7, 8, 9, 10], dtype=">i4")
    c = numpy.array([6, 7, 8, 9, 10], dtype=">i8")
    d = numpy.array([1, 2, 3, 4, 5], dtype=">i8")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)
        f["t"]["intBranch"].newbasket(b)
        f["t"]["int8Branch"].newbasket(c)
        f["t"]["int8Branch"].newbasket(d)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    intBranchdata = tree.AsMatrix(["intBranch"]).astype(">i4")
    int8Branchdata = tree.AsMatrix(["int8Branch"]).astype(">i8")
    for i in range(5):
        assert a[i] == intBranchdata[i]
        assert b[i] == intBranchdata[i+5]
        assert c[i] == int8Branchdata[i]
        assert d[i] == int8Branchdata[i+5]

def test_multi_tree_one_branch_multi_basket_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype=">i4")
    b = numpy.array([6, 7, 8, 9, 10], dtype=">i4")
    c = numpy.array([1, 2, 3, 4, 5, 6, 7, 8, 9], dtype=">i4")
    d = numpy.array([11, 12, 13, 14, 15, 16, 17, 18, 19], dtype=">i4")
    with uproot.recreate(filename, compression=None) as f:
        f["tree"] = tree
        f["tree"]["intBranch"].newbasket(c)
        f["tree"]["intBranch"].newbasket(d)
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)
        f["t"]["intBranch"].newbasket(b)

    f = uproot.open(filename)
    treedata1 = f["t"].array("intBranch")
    treedata2 = f["tree"].array("intBranch")
    for i in range(5):
        assert a[i] == treedata1[i]
        assert b[i] == treedata1[i+5]
    for i in range(9):
        assert c[i] == treedata2[i]
        assert d[i] == treedata2[i+9]

def test_multi_tree_one_branch_multi_basket(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype=">i4")
    b = numpy.array([6, 7, 8, 9, 10], dtype=">i4")
    c = numpy.array([1, 2, 3, 4, 5, 6, 7, 8, 9], dtype=">i4")
    d = numpy.array([11, 12, 13, 14, 15, 16, 17, 18, 19], dtype=">i4")
    with uproot.recreate(filename, compression=None) as f:
        f["tree"] = tree
        f["tree"]["intBranch"].newbasket(c)
        f["tree"]["intBranch"].newbasket(d)
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)
        f["t"]["intBranch"].newbasket(b)

    f = ROOT.TFile.Open(filename)
    t = f.Get("t")
    tree = f.Get("tree")
    treedata1 = t.AsMatrix().astype(">i4")
    treedata2 = tree.AsMatrix().astype(">i4")
    for i in range(5):
        assert a[i] == treedata1[i]
        assert b[i] == treedata1[i+5]
    for i in range(9):
        assert c[i] == treedata2[i]
        assert d[i] == treedata2[i+9]

# Not actually compressed
def test_tree_compression_empty(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch(">i4")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    with uproot.recreate(filename, compression=uproot.ZLIB(4)) as f:
        f["t"] = tree

    f = ROOT.TFile.Open(filename)
    assert f.Get("t").GetBranch("intBranch").GetName() == "intBranch"
    assert f.GetCompressionAlgorithm() == uproot.const.kZLIB
    assert f.GetCompressionLevel() == 4

# Not actually compressed
def test_tree_compression(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch(">i4")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype=">i4")
    with uproot.recreate(filename, compression=uproot.ZLIB(4)) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">i4")
    for i in range(5):
        assert a[i] == treedata[i]

def test_tree_branch_compression_only(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch(">i4", compression=uproot.ZLIB(4))
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype=">i4")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">i4")
    for i in range(5):
        assert a[i] == treedata[i]

def test_tree_branch_compression(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch(">i4", compression=uproot.ZLIB(4))
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype=">i4")
    with uproot.recreate(filename) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">i4")
    for i in range(5):
        assert a[i] == treedata[i]
    branch = tree.GetBranch("intBranch")
    assert branch.GetCompressionAlgorithm() == 1
    assert branch.GetCompressionLevel() == 4

def test_branch_compression_interface1(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch(">i8")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15], dtype=">i8")
    with uproot.recreate(filename, compression=uproot.ZLIB(4)) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">i8")
    for i in range(15):
        assert a[i] == treedata[i]
    branch = tree.GetBranch("intBranch")
    assert branch.GetCompressionAlgorithm() == 1
    assert branch.GetCompressionLevel() == 4

def test_branch_compression_interface1_diff_type(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch(">i4")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15], dtype=">i4")
    with uproot.recreate(filename, compression=uproot.ZLIB(4)) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">i4")
    for i in range(15):
        assert a[i] == treedata[i]
    branch = tree.GetBranch("intBranch")
    assert branch.GetCompressionAlgorithm() == 1
    assert branch.GetCompressionLevel() == 4

def test_branch_compression_interface2(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch(">i8", compression=uproot.ZLIB(4))
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15], dtype=">i8")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">i8")
    for i in range(15):
        assert a[i] == treedata[i]
    branch = tree.GetBranch("intBranch")
    assert branch.GetCompressionAlgorithm() == 1
    assert branch.GetCompressionLevel() == 4

def test_branch_compression_interface3(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch(">i8")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict, compression=uproot.ZLIB(4))
    a = numpy.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15], dtype=">i8")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">i8")
    for i in range(15):
        assert a[i] == treedata[i]
    branch = tree.GetBranch("intBranch")
    assert branch.GetCompressionAlgorithm() == 1
    assert branch.GetCompressionLevel() == 4

def test_many_basket_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch(">i4")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1], dtype=">i4")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        for i in range(101):
            f["t"]["intBranch"].newbasket(a)

    f = uproot.open(filename)
    tree = f["t"]
    treedata = tree.array("intBranch")
    for i in range(101):
        assert a[0] == treedata[i]

def test_many_basket(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch(">i4")
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1], dtype=">i4")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        for i in range(101):
            f["t"]["intBranch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">i4")
    for i in range(101):
        assert a[0] == treedata[i]

def test_tree_move_compress(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch(">i4", compression=uproot.ZLIB(4))
    branchdict = {"intBranch": b}
    tree = newtree(branchdict)
    a = numpy.array([1], dtype=">i4")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        for i in range(101):
            f["t"]["intBranch"].newbasket(a)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    treedata = tree.AsMatrix().astype(">i4")
    for i in range(101):
        assert a[0] == treedata[i]
    branch = tree.GetBranch("intBranch")
    assert branch.GetCompressionAlgorithm() == 1
    assert branch.GetCompressionLevel() == 4

def test_tree_renames(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = uproot.newbranch(">i4")
    branchdict = {"intBranch": b}
    tree = uproot.newtree(branchdict)
    a = numpy.array([1], dtype=">i4")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        for i in range(19):
            f["t"]["intBranch"].newbasket(a)

    f = uproot.open(filename)
    tree = f["t"]
    treedata = tree.array("intBranch")
    for i in range(19):
        assert a[0] == treedata[i]

def test_ttree_extend(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = uproot.newbranch(">i4")
    branchdict = {"intBranch": b, "intBranch2": b}
    tree = uproot.newtree(branchdict)
    with uproot.recreate(filename) as f:
        f["t"] = tree
        basket_add = {"intBranch": numpy.array([1, 2, 3, 4, 5]), "intBranch2": numpy.array([6, 7, 8, 9, 10])}
        f["t"].extend(basket_add)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    branch1 = tree.AsMatrix(["intBranch"])
    branch2 = tree.AsMatrix(["intBranch2"])
    branch1_test = numpy.array([1, 2, 3, 4, 5], dtype=">i4")
    branch2_test = numpy.array([6, 7, 8, 9, 10], dtype=">i4")
    for i in range(5):
        assert branch1[i] == branch1_test[i]
        assert branch2[i] == branch2_test[i]

def test_issue340(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0]
    with uproot.recreate(filename) as f:
        f["t"] = uproot.newtree({"normal": numpy.float64})
        f["t"].extend({"normal": a})

    t = uproot.open(filename)["t"]
    for i in range(10):
        assert t["normal"].basket(0)[i] == a[i]

def test_rdf(tmp_path):
    filename = join(str(tmp_path), "example.root")

    b = newbranch("int32")
    branchdict = {"intBranch": b, "intBranch2": b}
    tree = newtree(branchdict)
    a = numpy.array([1, 2, 3, 4, 5], dtype=">i4")
    b = numpy.array([11, 12, 13, 14, 15], dtype=">i4")
    with uproot.recreate(filename, compression=None) as f:
        f["t"] = tree
        f["t"]["intBranch"].newbasket(a)
        f["t"]["intBranch2"].newbasket(b)

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    rdf = ROOT.RDataFrame(tree)
    for i in range(5):
        assert a[i] == rdf.AsNumpy()["intBranch"][i]
        assert b[i] == rdf.AsNumpy()["intBranch2"][i]

def test_tree_cycle(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename) as f:
        f["t;1"] = uproot.newtree({"branch": "int32"})
        f["t;2"] = uproot.newtree({"branch": "int32"})
        f["t;1"].extend({"branch": numpy.array([1, 2, 3, 4, 5])})
        f["t"].extend({"branch": numpy.array([6, 7, 8, 9, 10])})

    f = ROOT.TFile.Open(filename)
    tree1 = f.Get("t;1")
    branch1 = tree1.AsMatrix(["branch"])
    tree2 = f.Get("t;2")
    branch2 = tree2.AsMatrix(["branch"])
    a = numpy.array([1, 2, 3, 4, 5], dtype="int32")
    b = numpy.array([6, 7, 8, 9, 10], dtype="int32")
    for i in range(5):
        assert branch1[i] == a[i]
        assert branch2[i] == b[i]

def test_large_compress(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename, uproot.ZLIB(5)) as f:
        f["a"] = "a" * ((2 ** 24) + 2000)
        f["b"] = "b" * ((2 ** 24) + 10)

    f = ROOT.TFile.Open(filename)
    assert str(f.Get("a")) == "a" * ((2 ** 24) + 2000)
    assert str(f.Get("b")) == "b" * ((2 ** 24) + 10)
    f.Close()

def test_large_compress_uproot(tmp_path):
    filename = join(str(tmp_path), "example.root")

    with uproot.recreate(filename, uproot.ZLIB(5)) as f:
        f["a"] = "a"*((2**24) + 2000)
        f["b"] = "b"*((2**24) + 10)

    f = uproot.open(filename)
    assert f["a"] == ("a"*((2**24) + 2000)).encode("utf-8")
    assert f["b"] == ("b"*((2**24) + 10)).encode("utf-8")

def test_tree_twodim(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = numpy.array([[0, 1, 2, 3],
                     [3, 4, 5, 6]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch": uproot.newbranch(numpy.dtype(">i4"), shape=a.shape)})
        f["t"].extend({"branch": a})

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    rdf = ROOT.RDataFrame(tree).AsNumpy()["branch"]
    for i in range(0, 2):
        for j in range(0, 4):
            assert a[i][j] == rdf[i][j]

def test_tree_threedim(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = numpy.array([[[0, 1, 2, 3],
                      [3, 4, 5, 6],
                      [90, 91, 91, 92]],
                     [[10, 11, 12, 13],
                      [13, 14, 15, 16],
                      [190, 191, 191, 192]]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch": uproot.newbranch(numpy.dtype(">i4"), shape=a.shape)})
        f["t"].extend({"branch": a})

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    rdf = ROOT.RDataFrame(tree).AsNumpy()["branch"]
    for i in range(2):
        test = numpy.array(rdf[i]).reshape(3, 4)
        for j in range(3):
            for k in range(4):
                assert a[i][j][k] == test[j][k]

def test_jagged_i4(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = awkward.fromiter([[0],
                          [1, 2],
                          [10, 11, 12]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch": uproot.newbranch(numpy.dtype(">i4"), size="n")})
        f["t"].extend({"branch": a, "n": [1, 2, 3]})

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    for i, event in enumerate(tree):
        assert(numpy.all([x for x in event.branch] == a[i]))

def test_jagged_uproot_i4(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = awkward.fromiter([[0],
                          [1, 2],
                          [10, 11, 12]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch": uproot.newbranch(numpy.dtype(">i4"), size="n")})
        f["t"].extend({"branch": a, "n": [1, 2, 3]})

    f = uproot.open(filename)
    array = f["t"].array(["branch"])
    for i in range(len(array)):
        for j in range(len(array[i])):
            assert(array[i][j] == a[i][j])

#Need to use C++ code to read out because of bug in PyROOT layer (of Conda ROOT build?)
def test_jagged_i8(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = awkward.fromiter([[0],
                          [1, 2],
                          [10, 11, 12]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch": uproot.newbranch(numpy.dtype(">i8"), size="n")})
        f["t"].extend({"branch": a, "n": [1, 2, 3]})

    ROOT.gInterpreter.Declare("""
        void assertint(bool &flag, char* filename) {
            TFile *f = new TFile(filename);
            Long64_t x;
            Int_t num;
            auto tree = f->Get<TTree>("t");
            auto n = tree->GetBranch("n");
            auto branch = tree->GetBranch("branch");
            n->SetAddress(&num);
            branch->SetAddress(&x);
            Long64_t values[3][3] = {{0,0,0}, {1, 2, 0}, {10, 11, 12}};
            for (int i=0; i<tree->GetEntries(); i++) {
                tree->GetEvent(i);
                for (int j=0; j<num; j++) {
                    if (values[i][j] != x+j)
                        flag = false;
                }
            }
        }""")

    flag = ctypes.c_bool(True)
    ROOT.assertint(flag, filename)
    assert(flag)

def test_jagged_uproot_i8(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = awkward.fromiter([[0],
                          [1, 2],
                          [10, 11, 12]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch": uproot.newbranch(numpy.dtype(">i8"), size="n")})
        f["t"].extend({"branch": a, "n": [1, 2, 3]})

    f = uproot.open(filename)
    array = f["t"].array(["branch"])
    for i in range(len(array)):
        for j in range(len(array[i])):
            assert(array[i][j] == a[i][j])

def test_jagged_f8(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = awkward.fromiter([[0],
                          [1, 2],
                          [10, 11, 12]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch": uproot.newbranch(numpy.dtype(">f8"), size="n")})
        f["t"].extend({"branch": a, "n": [1, 2, 3]})

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    for i, event in enumerate(tree):
        assert(numpy.all([x for x in event.branch] == a[i]))

def test_jagged_uproot_f8(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = awkward.fromiter([[0],
                          [1, 2],
                          [10, 11, 12]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch": uproot.newbranch(numpy.dtype(">f8"), size="n")})
        f["t"].extend({"branch": a, "n": [1, 2, 3]})

    f = uproot.open(filename)
    array = f["t"].array(["branch"])
    for i in range(len(array)):
        for j in range(len(array[i])):
            assert(array[i][j] == a[i][j])

def test_jagged_f4(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = awkward.fromiter([[0],
                          [1, 2],
                          [10, 11, 12]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch": uproot.newbranch(numpy.dtype(">f4"), size="n")})
        f["t"].extend({"branch": a, "n": [1, 2, 3]})

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    for i, event in enumerate(tree):
        assert(numpy.all([x for x in event.branch] == a[i]))

def test_jagged_uproot_f4(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = awkward.fromiter([[0],
                          [1, 2],
                          [10, 11, 12]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch": uproot.newbranch(numpy.dtype(">f4"), size="n")})
        f["t"].extend({"branch": a, "n": [1, 2, 3]})

    f = uproot.open(filename)
    array = f["t"].array(["branch"])
    for i in range(len(array)):
        for j in range(len(array[i])):
            assert(array[i][j] == a[i][j])

def test_jagged_i2(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = awkward.fromiter([[0],
                          [1, 2],
                          [10, 11, 12]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch": uproot.newbranch(numpy.dtype(">i2"), size="n")})
        f["t"].extend({"branch": a, "n": [1, 2, 3]})

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    for i, event in enumerate(tree):
        assert(numpy.all([x for x in event.branch] == a[i]))

def test_jagged_uproot_i2(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = awkward.fromiter([[0],
                          [1, 2],
                          [10, 11, 12]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch": uproot.newbranch(numpy.dtype(">i2"), size="n")})
        f["t"].extend({"branch": a, "n": [1, 2, 3]})

    f = uproot.open(filename)
    array = f["t"].array(["branch"])
    for i in range(len(array)):
        for j in range(len(array[i])):
            assert(array[i][j] == a[i][j])

def test_jagged_i2_multiple_sametype(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = awkward.fromiter([[0],
                          [1, 2]])

    b = awkward.fromiter([[3],
                          [7, 12]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch1": uproot.newbranch(numpy.dtype(">i2"), size="n"),
                                 "branch2": uproot.newbranch(numpy.dtype(">i2"), size="n")})
        f["t"].extend({"branch1": a,
                       "branch2": b,
                       "n": [1, 2]})

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    for i, event in enumerate(tree):
        assert(numpy.all([x for x in event.branch1] == a[i]))
        assert(numpy.all([x for x in event.branch2] == b[i]))

def test_jagged_multiple_difftype(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = awkward.fromiter([[0],
                          [1, 2]])

    b = awkward.fromiter([[3],
                          [7, 12]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch1": uproot.newbranch(numpy.dtype(">i2"), size="n"),
                                 "branch2": uproot.newbranch(numpy.dtype(">i4"), size="n")})
        f["t"].extend({"branch1": a,
                       "branch2": b,
                       "n": [1, 2]})

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    for i, event in enumerate(tree):
        assert(numpy.all([x for x in event.branch1] == a[i]))
        assert(numpy.all([x for x in event.branch2] == b[i]))

def test_jagged_i2_multiple_difflen(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = awkward.fromiter([[0],
                          [1, 2]])

    b = awkward.fromiter([[3],
                          [10, 11, 12]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch1": uproot.newbranch(numpy.dtype(">i2"), size="n1"),
                                 "branch2": uproot.newbranch(numpy.dtype(">i2"), size="n2")})
        f["t"].extend({"branch1": a,
                       "n1": [1, 2],
                       "branch2": b,
                       "n2": [1, 3]})

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    for i, event in enumerate(tree):
        assert(numpy.all([x for x in event.branch1] == a[i]))
        assert(numpy.all([x for x in event.branch2] == b[i]))

def test_jagged_i4_manybasket(tmp_path):
    filename = join(str(tmp_path), "example.root")

    a = awkward.fromiter([[0],
                          [1, 2],
                          [10, 11, 12]])
    b = awkward.fromiter([[10],
                          [11, 12]])
    tester = awkward.fromiter([[0],
                               [1, 2],
                               [10, 11, 12],
                               [10],
                               [11, 12]])

    with uproot.recreate(filename, compression=None) as f:
        f["t"] = uproot.newtree({"branch": uproot.newbranch(numpy.dtype(">i4"), size="n")})
        f["t"].extend({"branch": a, "n": [1, 2, 3]})
        f["t"].extend({"branch": b, "n": [1, 2]})

    f = ROOT.TFile.Open(filename)
    tree = f.Get("t")
    for i, event in enumerate(tree):
        assert(numpy.all([x for x in event.branch] == tester[i]))



#!/usr/bin/env python

# BSD 3-Clause License; see https://github.com/scikit-hep/uproot/blob/master/LICENSE

import sys
import os.path

from setuptools import find_packages
from setuptools import setup

def get_version():
    g = {}
    exec(open(os.path.join("uproot", "version.py")).read(), g)
    return g["__version__"]

def get_description():
    description = open("README.rst", "rb").read().decode("utf8", "ignore")

    before = """.. image:: https://raw.githubusercontent.com/scikit-hep/uproot/master/docs/source/logo-300px.png
   :alt: uproot
   :target: https://github.com/scikit-hep/uproot

"""

    start = description.index(".. inclusion-marker-1-5-do-not-remove")
    stop = description.index(".. inclusion-marker-3-do-not-remove")
    middle = description[start:stop].strip()
    start_replaceplots = middle.index(".. inclusion-marker-replaceplots-start")
    stop_replaceplots = middle.index(".. inclusion-marker-replaceplots-stop") + len(".. inclusion-marker-replaceplots-stop")
    middle = middle[:start_replaceplots] + """
.. image:: https://raw.githubusercontent.com/scikit-hep/uproot/master/docs/root-none-muon.png
   :width: 350 px
.. image:: https://raw.githubusercontent.com/scikit-hep/uproot/master/docs/rootnumpy-none-muon.png
   :width: 350 px
""" + middle[stop_replaceplots:]

    after = """

Tutorial
========

See the `project homepage <https://github.com/scikit-hep/uproot>`__ for a `tutorial <https://github.com/scikit-hep/uproot#tutorial>`__.

Run `that tutorial <https://mybinder.org/v2/gh/scikit-hep/uproot/master?urlpath=lab/tree/binder%2Ftutorial.ipynb>`__ on Binder.

**Tutorial contents:**

* `Introduction <https://github.com/scikit-hep/uproot#introduction>`__
* `What is uproot? <https://github.com/scikit-hep/uproot#what-is-uproot>`__
* `Exploring a file <https://github.com/scikit-hep/uproot#exploring-a-file>`__

  - `Compressed objects in ROOT files <https://github.com/scikit-hep/uproot#compressed-objects-in-root-files>`__
  - `Exploring a TTree <https://github.com/scikit-hep/uproot#exploring-a-ttree>`__
  - `Some terminology <https://github.com/scikit-hep/uproot#some-terminology>`__

* `Reading arrays from a TTree <https://github.com/scikit-hep/uproot#reading-arrays-from-a-ttree>`__
* `Caching data <https://github.com/scikit-hep/uproot#caching-data>`__

  - `Automatically managed caches <https://github.com/scikit-hep/uproot#automatically-managed-caches>`__
  - `Caching at all levels of abstraction <https://github.com/scikit-hep/uproot#caching-at-all-levels-of-abstraction>`__

* `Lazy arrays <https://github.com/scikit-hep/uproot#lazy-arrays>`__

  - `Lazy array of many files <https://github.com/scikit-hep/uproot#lazy-array-of-many-files>`__
  - `Lazy arrays with caching <https://github.com/scikit-hep/uproot#lazy-arrays-with-caching>`__
  - `Lazy arrays as lightweight skims <https://github.com/scikit-hep/uproot#lazy-arrays-as-lightweight-skims>`__
  - `Lazy arrays in Dask <https://github.com/scikit-hep/uproot#lazy-arrays-in-dask>`__

* `Iteration <https://github.com/scikit-hep/uproot#iteration>`__

  - `Filenames and entry numbers while iterating <https://github.com/scikit-hep/uproot#filenames-and-entry-numbers-while-iterating>`__
  - `Limiting the number of entries to be read <https://github.com/scikit-hep/uproot#limiting-the-number-of-entries-to-be-read>`__
  - `Controlling lazy chunk and iteration step sizes <https://github.com/scikit-hep/uproot#controlling-lazy-chunk-and-iteration-step-sizes>`__
  - `Caching and iteration <https://github.com/scikit-hep/uproot#caching-and-iteration>`__

* `Changing the output container type <https://github.com/scikit-hep/uproot#changing-the-output-container-type>`__
* `Filling Pandas DataFrames <https://github.com/scikit-hep/uproot#filling-pandas-dataframes>`__
* `Selecting and interpreting branches <https://github.com/scikit-hep/uproot#selecting-and-interpreting-branches>`__

  - `TBranch interpretations <https://github.com/scikit-hep/uproot#tbranch-interpretations>`__
  - `Reading data into a preexisting array <https://github.com/scikit-hep/uproot#reading-data-into-a-preexisting-array>`__
  - `Passing many new interpretations in one call <https://github.com/scikit-hep/uproot#passing-many-new-interpretations-in-one-call>`__
  - `Multiple values per event: fixed size arrays <https://github.com/scikit-hep/uproot#multiple-values-per-event-fixed-size-arrays>`__
  - `Multiple values per event: leaf-lists <https://github.com/scikit-hep/uproot#multiple-values-per-event-leaf-lists>`__
  - `Multiple values per event: jagged arrays <https://github.com/scikit-hep/uproot#multiple-values-per-event-jagged-arrays>`__
  - `Jagged array performance <https://github.com/scikit-hep/uproot#jagged-array-performance>`__
  - `Special physics objects: Lorentz vectors <https://github.com/scikit-hep/uproot#special-physics-objects-lorentz-vectors>`__
  - `Variable-width values: strings <https://github.com/scikit-hep/uproot#variable-width-values-strings>`__
  - `Arbitrary objects in TTrees <https://github.com/scikit-hep/uproot#arbitrary-objects-in-ttrees>`__
  - `Doubly nested jagged arrays (i.e. std::vector<std::vector<T>>) <https://github.com/scikit-hep/uproot#doubly-nested-jagged-arrays-ie-stdvectorstdvectort>`__

* `Parallel array reading <https://github.com/scikit-hep/uproot#parallel-array-reading>`__
* `Histograms, TProfiles, TGraphs, and others <https://github.com/scikit-hep/uproot#histograms-tprofiles-tgraphs-and-others>`__
* `Creating and writing data to ROOT files <https://github.com/scikit-hep/uproot#creating-and-writing-data-to-root-files>`__

  - `Writing histograms <https://github.com/scikit-hep/uproot#writing-histograms>`__
  - `Writing TTrees <https://github.com/scikit-hep/uproot#writing-ttrees>`__

Reference documentation
=======================

* `Opening files <http://uproot.readthedocs.io/en/latest/opening-files.html>`__

  - `uproot.open <http://uproot.readthedocs.io/en/latest/opening-files.html#uproot-open>`__
  - `uproot.xrootd <http://uproot.readthedocs.io/en/latest/opening-files.html#uproot-xrootd>`__
  - `uproot.http <http://uproot.readthedocs.io/en/latest/opening-files.html#uproot-http>`__
  - `uproot.iterate <http://uproot.readthedocs.io/en/latest/opening-files.html#uproot-iterate>`__
  - `uproot.pandas.iterate <http://uproot.readthedocs.io/en/latest/opening-files.html#uproot-pandas-iterate>`__
  - `uproot.lazyarray(s) <http://uproot.readthedocs.io/en/latest/opening-files.html#uproot-lazyarray-and-lazyarrays>`__
  - `uproot.daskarray/daskframe <http://uproot.readthedocs.io/en/latest/opening-files.html#uproot-daskarray-and-daskframe>`__
  - `uproot.numentries <http://uproot.readthedocs.io/en/latest/opening-files.html#uproot-numentries>`__

* `ROOT I/O <http://uproot.readthedocs.io/en/latest/root-io.html>`__

  - `uproot.rootio.ROOTDirectory <http://uproot.readthedocs.io/en/latest/root-io.html#uproot-rootio-rootdirectory>`__
  - `uproot.rootio.ROOTObject <http://uproot.readthedocs.io/en/latest/root-io.html#uproot-rootio-rootobject>`__
  - `uproot.rootio.ROOTStreamedObject <http://uproot.readthedocs.io/en/latest/root-io.html#uproot-rootio-rootstreamedobject>`__

* `TTree Handling <http://uproot.readthedocs.io/en/latest/ttree-handling.html>`__

  - `uproot.tree.TTreeMethods <http://uproot.readthedocs.io/en/latest/ttree-handling.html#uproot-tree-ttreemethods>`__
  - `uproot.tree.TBranchMethods <http://uproot.readthedocs.io/en/latest/ttree-handling.html#uproot-tree-tbranchmethods>`__

* `Interpretation <http://uproot.readthedocs.io/en/latest/interpretation.html>`__
* `Caches <http://uproot.readthedocs.io/en/latest/caches.html>`__
* `Parallel I/O <http://uproot.readthedocs.io/en/latest/parallel-io.html>`__
"""
    return before + middle + after

setup(name = "uproot",
      version = get_version(),
      packages = find_packages(exclude = ["tests"]),
      scripts = [],
      description = "ROOT I/O in pure Python and Numpy.",
      long_description = get_description(),
      author = "Jim Pivarski (IRIS-HEP)",
      author_email = "pivarski@princeton.edu",
      maintainer = "Jim Pivarski (IRIS-HEP)",
      maintainer_email = "pivarski@princeton.edu",
      url = "https://github.com/scikit-hep/uproot",
      download_url = "https://github.com/scikit-hep/uproot/releases",
      license = "BSD 3-clause",
      test_suite = "tests",
      python_requires = ">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*",
      install_requires = ["numpy>=1.13.1", "awkward>=0.12.0,<1.0", "uproot-methods>=0.7.0", "cachetools"],
      setup_requires = ["pytest-runner"],
      extras_require = {
          "testing": ["pytest>=3.9", "pkgconfig", "lz4", "zstandard", 'backports.lzma;python_version<"3.3"', "xxhash", "mock", "requests"],
          "compress": ["lz4", "zstandard", 'backports.lzma;python_version<"3.3"', "xxhash"],
      },
      classifiers = [
          "Development Status :: 5 - Production/Stable",
          "Intended Audience :: Developers",
          "Intended Audience :: Information Technology",
          "Intended Audience :: Science/Research",
          "License :: OSI Approved :: BSD License",
          "Operating System :: MacOS",
          "Operating System :: POSIX",
          "Operating System :: Unix",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3.5",
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7",
          "Programming Language :: Python :: 3.8",
          "Topic :: Scientific/Engineering",
          "Topic :: Scientific/Engineering :: Information Analysis",
          "Topic :: Scientific/Engineering :: Mathematics",
          "Topic :: Scientific/Engineering :: Physics",
          "Topic :: Software Development",
          "Topic :: Utilities",
          ],
      platforms = "Any",
      )



#!/usr/bin/env python

# BSD 3-Clause License; see https://github.com/scikit-hep/uproot/blob/master/LICENSE

import pytest
import mock
HTTPError = pytest.importorskip('requests.exceptions').HTTPError

import uproot

FILE = "foriter"
LOCAL = "tests/samples/{FILE}.root".format(FILE=FILE)
URL = "http://scikit-hep.org/uproot/examples/{FILE}.root".format(FILE=FILE)
URL_AUTH = "http://scikit-hep.org/uproot/authentication/{FILE}.root".format(FILE=FILE)
AUTH = ("scikit-hep", "uproot")

def mock_get_local_instead_of_http(url="", headers={}, auth=None, **kwargs):
    class MockResponse:
        def __init__(self, status_code):
            self.status_code = status_code
            if self.status_code == 200:
                with open(LOCAL, "rb") as f:
                    self.content = f.read()
                self.headers = {"Content-Range": str(len(self.content))}

        def raise_for_status(self):
            if self.status_code == 401:  # Authentication Error
                raise HTTPError
            elif self.status_code == 200:  # Ok
                pass

    if url == URL:
        return MockResponse(200)
    elif url == URL_AUTH and auth == None:
        return MockResponse(401)
    elif url == URL_AUTH and auth == AUTH:
        return MockResponse(200)
    elif url == URL_AUTH:
        return MockResponse(401)

@mock.patch("requests.get", mock_get_local_instead_of_http)
class Test(object):
    def test_no_auth_needed_no_auth(self):
        f = uproot.open(URL)
        assert type(f) == uproot.rootio.ROOTDirectory

    def test_no_auth_needed_with_auth(self):
        f = uproot.open(URL, httpsource={"auth": AUTH})
        assert type(f) == uproot.rootio.ROOTDirectory

    def test_auth_needed_no_auth(self):
        with pytest.raises(HTTPError):
            f = uproot.open(URL_AUTH)

    def test_auth_needed_correct_auth(self):
        f = uproot.open(URL_AUTH, httpsource={"auth": AUTH})
        assert type(f) == uproot.rootio.ROOTDirectory

    def test_auth_needed_wrong_auth(self):
        with pytest.raises(HTTPError):
            f = uproot.open(URL_AUTH, httpsource={"auth": ("", "")})



#!/usr/bin/env python

# BSD 3-Clause License; see https://github.com/scikit-hep/uproot/blob/master/LICENSE

import pytest
try:
    import lzma
except ImportError:
    lzma = pytest.importorskip('backports.lzma')
lz4 = pytest.importorskip('lz4')
zstandard = pytest.importorskip('zstandard')
import uproot


class Test(object):
    def test_compression_identity(self):
        assert uproot.open("tests/samples/Zmumu-zlib.root").compression.algoname == "zlib"
        assert uproot.open("tests/samples/Zmumu-zlib.root").compression.level == 4

        assert uproot.open("tests/samples/Zmumu-lzma.root").compression.algoname == "lzma"
        assert uproot.open("tests/samples/Zmumu-lzma.root").compression.level == 4

        assert uproot.open("tests/samples/Zmumu-lz4.root").compression.algoname == "lz4"
        assert uproot.open("tests/samples/Zmumu-lz4.root").compression.level == 4

        assert uproot.open("tests/samples/Zmumu-zstd.root").compression.algoname == "zstd"
        assert uproot.open("tests/samples/Zmumu-zstd.root").compression.level == 5

        assert uproot.open("tests/samples/Zmumu-uncompressed.root").compression.level == 0

        assert uproot.open("tests/samples/HZZ-zlib.root").compression.algoname == "zlib"
        assert uproot.open("tests/samples/HZZ-zlib.root").compression.level == 4

        assert uproot.open("tests/samples/HZZ-lzma.root").compression.algoname == "lzma"
        assert uproot.open("tests/samples/HZZ-lzma.root").compression.level == 4

        assert uproot.open("tests/samples/HZZ-lz4.root").compression.algoname == "lz4"
        assert uproot.open("tests/samples/HZZ-lz4.root").compression.level == 4

        assert uproot.open("tests/samples/HZZ-zstd.root").compression.algoname == "zstd"
        assert uproot.open("tests/samples/HZZ-zstd.root").compression.level == 5

        assert uproot.open("tests/samples/HZZ-uncompressed.root").compression.level == 0

    def test_compression_keys(self):
        keys = [(n, cls._classname) for n, cls in uproot.open("tests/samples/Zmumu-uncompressed.root").allclasses()]
        assert [(n, cls._classname) for n, cls in uproot.open("tests/samples/Zmumu-zlib.root").allclasses()] == keys
        assert [(n, cls._classname) for n, cls in uproot.open("tests/samples/Zmumu-lzma.root").allclasses()] == keys
        assert [(n, cls._classname) for n, cls in uproot.open("tests/samples/Zmumu-lz4.root").allclasses()] == keys
        assert [(n, cls._classname) for n, cls in uproot.open("tests/samples/Zmumu-zstd.root").allclasses()] == keys

        keys = [(n, cls._classname) for n, cls in uproot.open("tests/samples/HZZ-uncompressed.root").allclasses()]
        assert [(n, cls._classname) for n, cls in uproot.open("tests/samples/HZZ-zlib.root").allclasses()] == keys
        assert [(n, cls._classname) for n, cls in uproot.open("tests/samples/HZZ-lzma.root").allclasses()] == keys
        assert [(n, cls._classname) for n, cls in uproot.open("tests/samples/HZZ-lz4.root").allclasses()] == keys
        assert [(n, cls._classname) for n, cls in uproot.open("tests/samples/HZZ-zstd.root").allclasses()] == keys

    def test_compression_branches(self):
        branches = list(uproot.open("tests/samples/Zmumu-uncompressed.root")["events"].keys())
        assert list(uproot.open("tests/samples/Zmumu-zlib.root")["events"].keys()) == branches
        assert list(uproot.open("tests/samples/Zmumu-lzma.root")["events"].keys()) == branches
        assert list(uproot.open("tests/samples/Zmumu-lz4.root")["events"].keys()) == branches
        assert list(uproot.open("tests/samples/Zmumu-zstd.root")["events"].keys()) == branches

        branches = list(uproot.open("tests/samples/HZZ-uncompressed.root")["events"].keys())
        assert list(uproot.open("tests/samples/HZZ-zlib.root")["events"].keys()) == branches
        assert list(uproot.open("tests/samples/HZZ-lzma.root")["events"].keys()) == branches
        assert list(uproot.open("tests/samples/HZZ-lz4.root")["events"].keys()) == branches
        assert list(uproot.open("tests/samples/HZZ-zstd.root")["events"].keys()) == branches

    def test_compression_content1(self):
        for name, array in uproot.open("tests/samples/Zmumu-uncompressed.root")["events"].arrays(["Type", "Event", "E1", "px1", "Q1", "M"]).items():
            array = array.tolist()
            assert uproot.open("tests/samples/Zmumu-zlib.root")["events"].array(name).tolist() == array
            assert uproot.open("tests/samples/Zmumu-lzma.root")["events"].array(name).tolist() == array
            assert uproot.open("tests/samples/Zmumu-lz4.root")["events"].array(name).tolist() == array
            assert uproot.open("tests/samples/Zmumu-zstd.root")["events"].array(name).tolist() == array

    def test_compression_content2(self):
        array = uproot.open("tests/samples/HZZ-uncompressed.root")["events"].array("Electron_Px").tolist()
        assert uproot.open("tests/samples/HZZ-zlib.root")["events"].array("Electron_Px").tolist() == array
        assert uproot.open("tests/samples/HZZ-lzma.root")["events"].array("Electron_Px").tolist() == array
        assert uproot.open("tests/samples/HZZ-lz4.root")["events"].array("Electron_Px").tolist() == array
        assert uproot.open("tests/samples/HZZ-zstd.root")["events"].array("Electron_Px").tolist() == array



#!/usr/bin/env python

# BSD 3-Clause License; see https://github.com/scikit-hep/uproot/blob/master/LICENSE

from __future__ import absolute_import

import struct

class Cursor(object):
    def __init__(self, index):
        self.index = index

    def skip(self, numbytes):
        self.index += numbytes

    def update_fields(self, sink, format, *args):
        sink.write(format.pack(*args), self.index)

    def write_fields(self, sink, format, *args):
        self.update_fields(sink, format, *args)
        self.index += format.size

    def put_fields(self, format, *args):
        self.index += format.size
        return format.pack(*args)

    @staticmethod
    def length_string(string):
        if len(string) < 255:
            return len(string) + 1
        else:
            return len(string) + 5

    @staticmethod
    def length_strings(strings):
        return sum(Cursor.length_string(x) for x in strings)

    _format_byte = struct.Struct("B")
    _format_byteint = struct.Struct(">Bi")
    def update_string(self, sink, data):
        if len(data) < 255:
            sink.write(self._format_byte.pack(len(data)), self.index)
            sink.write(data, self.index + 1)
        else:
            sink.write(self._format_byteint.pack(255, len(data)), self.index)
            sink.write(data, self.index + 5)

    def write_string(self, sink, data):
        self.update_string(sink, data)
        self.index += self.length_string(data)

    def put_string(self, data):
        self.index += self.length_string(data)
        if len(data) < 255:
            return self._format_byte.pack(len(data)) + data
        else:
            return self._format_byteint.pack(255, len(data)) + data

    def update_cstring(self, sink, data):
        sink.write(data, self.index)
        sink.write(b"\x00")

    def write_cstring(self, sink, data):
        self.update_cstring(sink, data)
        self.index += len(data) + 1

    def put_cstring(self, data):
        self.index += len(data) + 1
        return data.encode("utf-8") + b"\x00"

    def update_data(self, sink, data):
        sink.write(data, self.index)

    def write_data(self, sink, data):
        self.update_data(sink, data)
        self.index += len(data)

    def put_data(self, data):
        self.index += len(data)
        return data

    def put_array(self, data):
        self.index += data.nbytes
        return data.tostring()

    def update_array(self, sink, data):
        sink.write(data.tostring(), self.index)

    def write_array(self, sink, data):
        self.update_array(sink, data)
        self.index += data.nbytes



#!/usr/bin/env python

# BSD 3-Clause License; see https://github.com/scikit-hep/uproot/blob/master/LICENSE

from __future__ import absolute_import

class Sink(object):
    pass

class FileSink(Sink):
    def __init__(self, path):
        self._path = path
        self._sink = open(path, "wb+")
        # self._pos = 0

    def write(self, data, pos):
        self._sink.seek(pos)
        self._sink.write(data)

    # def write(self, data, pos):
    #     if self._sink.tell() != pos:
    #         self._sink.seek(pos)
    #     self._sink.write(data)

    # def write(self, data, pos):
    #     if self._pos != pos:
    #         self._sink.seek(pos)
    #     self._sink.write(data)
    #     self._pos += len(data)    # needs self._pos

    @property
    def closed(self):
        return self._sink.closed

    def close(self):
        if not self._sink.closed:
            self._sink.close()

    def flush(self):
        if not self._sink.closed:
            self._sink.flush()
